import { ConnectConfig } from "ssh2";
import { NilCallback } from "util.toolbox";
export interface ScaffoldOpts extends ConnectConfig {
    stub?: boolean;
    hostname?: string;
    privateKeyFile?: string;
    publicKeyFile?: string;
    publicKey?: string;
    timeout?: number;
}
export interface CommandOpts {
    cwd?: string;
    delay?: number;
    group?: string;
    mode?: string;
    owner?: string;
    recursive?: boolean;
    sudo?: boolean;
    verbose?: boolean;
    shell?: string;
}
/** an instance of the Scaffold class */
export declare class Scaffold {
    private _debug;
    private _cmds;
    private _config;
    private _history;
    private _local;
    private _semaphore;
    /**
     * The constructor function for the scaffolding.  This takes a single parameter
     * that represents the configuration required to connect to a remote host using
     * SSH.  If the config is empty, then the commands are all executed on the
     * local host instead.
     * @param [opts] {ScaffoldOpts} holds SSH connection information from config.json
     * @constructor
     */
    constructor(opts?: ScaffoldOpts);
    /**
     * Runs a command on a remote or local server.
     * @param cmd {string} the command to run
     * @param opts {object} an object that holds the parameters used to run
     * this command.
     *
     *   - cwd: current working directory
     *   - sudo: a boolean that determines if sudo should be used
     *   - delay: how many seconds to delay after the call.
     */
    run(cmd: string, opts?: CommandOpts): Scaffold;
    /**
     * A sugar wrapper for calling run with sudo wrapped around it.  See "run" for
     * the list of parameters in the args object.
     * @param cmd {string} the sudo command to execute
     * @param opts {object} an object that holds the parameters to the sudo call.
     */
    sudo(cmd: string, opts?: CommandOpts): Scaffold;
    /**
     * Takes a text file and puts it on the target remote machine.
     * @param lfile {string} the local file name to send to the remote
     * @param rfile {string} the name of the remote file to send
     * @param opts {object} the list of optional arguments  these include:
     *
     *   - mode: the file mode octet for the file being placed
     *   - owner: the owner that should be set for this file
     *   - group: linux group permission to set on file.
     *
     * @returns {Scaffold} a reference to this object for chaining.
     */
    put(lfile: string, rfile: string, opts?: CommandOpts): this;
    /**
     * Creates a directory on the remote server.
     * @param directory {string} the directory to factory on the remote server
     *
     * @param opts {object} the list of optional arguments  these include:
     *
     *   - mode: the file mode octet for the file being placed
     *   - owner: the owner that should be set for this file
     *   - group: linux group permission to set on file.
     *
     * @returns {Scaffold} a reference to this object for chaining.
     */
    mkdir(directory: string, opts?: CommandOpts): this;
    /**
     * Copies a file from one location to the other (on the same server).  Use put
     * to move a file from locatl to remote.  This just moves things around on the
     * same server.
     * @param src {string} the source file to move
     * @param dst {string} the destination location
     * @param [opts] {object} the optional arguments object
     *
     *   - recursive: {boolean} if true, use the -r flag on the copy
     *   - sudo: {boolean} if true use sudo, otherwise regular copy
     *
     * @returns {Scaffold} a reference to this object for chaining.
     */
    copy(src: string, dst: string, opts?: CommandOpts): this;
    /**
     * Starts the processing of the command queue.
     * @param [opts] {CommandOpts} a set of commands used to process this queue.
     *
     *     - verbose: {boolean} if true, then print more output, otherwise silent
     *     - shell: {string} the shell that the command should be run against.  This
     *       is only relevant for the local processor.   The remote processor runs
     *       with the shell of the authenticated in user.
     *
     * @param [cb] {Function} a callback function that is executed when this process
     * completes.  It will be executed on success or failure.
     */
    go(opts?: CommandOpts | any, cb?: NilCallback): void;
    get local(): boolean;
    get history(): string[];
    /**
     * Runs the given command queue on the local host.
     * @param opts {CommandOpts} the options that are used to run all commands
     * in the queue.
     * @param cb {Function} a callback function that is executed when the
     * remote execution finishes or has an error.
     * @param self {Scaffold} a reference to the Scaffold instance.
     * @returns the result of the given callback function.
     * @private
     */
    private runLocal;
    /**
     * Runs the given command queue on the remote server.  It uses the ssh2
     * library to connnect and execute the commands on the server.
     * @param opts {CommandOpts} the options that are used to run all commands
     * in the queue.
     * @param cb {Function} a callback function that is executed when the
     * remote execution finishes or has an error.
     * @param self {Scaffold} a reference to the Scaffold instance.
     * @returns the result of the given callback function.
     * @private
     */
    private runRemote;
}
